use strict;
use warnings;
use Test::More tests => 6;

use_ok 'Text::KDL::XS';
use_ok 'Text::KDL::XS::Parser';
use_ok 'Text::KDL::XS::Document';
use_ok 'Text::KDL::XS::Node';
use_ok 'Text::KDL::XS::Value';
use_ok 'Text::KDL::XS::Emitter';
